//importing libraries:
#include <WiFi.h>
#include <HTTPClient.h>



//used device pins:
const int pressureSensorPin = 35;
const int currentSensorPin = 34;
const int valvePin = 27;

float pressureReleaseValue = 2.0;

float pressureInput;
float pressureCalculated;
float maxPressure;
float averagePressureCalibrator;

float currentInput;
float currentCalculated;
float maxCurrent;

float averageCurrent;

//wifi cridentials to make connection to the internet:
const char* ssid   = "*sensored*";
const char* password = "*sensored*";

//server upload adress:
const char* serverName = "http://earthfarts.ga/post-esp-data.php";

//unique API key so not everyone can get access:
String apiKeyValue = "*sensored*";

//custom sensor data:
String sensorName = "Geyser_001";
String sensorLocation = "Enschede";


void setup() {
  pinMode(valvePin, OUTPUT);
  Serial.begin(115200);
  WiFi.begin(ssid, password);
  Serial.println("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("trying to connect...");
  }
  Serial.println("");
  Serial.print("Connected to WiFi network with IP Address: ");
  Serial.println(WiFi.localIP());

  averagePressureCalibrator = 0;
  for (int i = 0; i < 100; i++) {
    averagePressureCalibrator += analogRead(pressureSensorPin);
  }
  averagePressureCalibrator = averagePressureCalibrator / 100;
}

void loop() {
  //while the pressure is too low this code runs:
  pressureCalculated = 0;
  while (pressureCalculated <= pressureReleaseValue) {
    pressureInput = analogRead(pressureSensorPin);
    pressureCalculated = pressureInput / averagePressureCalibrator;
    Serial.print("pressure: ");
    Serial.print(pressureCalculated, 2);
    Serial.println(" bar");
    delay(250);
  }

  //after the valve opening the pressure will be measured a few extra times.
  maxPressure = 0;
  for (int i = 0; i <= 500; i++) {
    pressureInput = analogRead(pressureSensorPin);
    pressureCalculated = pressureInput / averagePressureCalibrator;
    if (pressureCalculated > maxPressure) {
      maxPressure = pressureCalculated;
    }
    delay(5);
  }

  //this opens the valve:
  digitalWrite(valvePin, HIGH);

  //current will be sensed for 7.5 seconds:
  averageCurrent = 0;
  maxCurrent = 0;
  for (int i = 0; i < 5000; i++) {
    currentInput = analogRead(currentSensorPin);
    currentCalculated = (currentInput - 1844) / 112.0;
    //Serial.println(currentCalculated);
    averageCurrent += currentCalculated;
    if (currentCalculated > maxCurrent) {
      maxCurrent = currentCalculated;
    }
    delay(3);
  }

  //calculating average current generated:
  averageCurrent = averageCurrent / 1500;

  //valve gets closed again:
  digitalWrite(valvePin, LOW);


  //upload to database code:
  if (WiFi.status() == WL_CONNECTED) {
    WiFiClient client;
    HTTPClient http;

    http.begin(client, serverName);
    http.addHeader("Content-Type", "application/x-www-form-urlencoded");

    String httpRequestData = "api_key=" + apiKeyValue + "&sensor=" + sensorName
                             + "&location=" + sensorLocation + "&value1=" + maxPressure
                             + "&value2=" + maxCurrent + "&value3=" + averageCurrent + "";
    Serial.print("httpRequestData: ");
    Serial.println(httpRequestData);

    int httpResponseCode = http.POST(httpRequestData);

    if (httpResponseCode > 0) {
      Serial.print("HTTP Response code: ");
      Serial.println(httpResponseCode);
    }
    else {
      Serial.print("Error code: ");
      Serial.println(httpResponseCode);
    }

    http.end();
  }
  else {
    Serial.println("WiFi Disconnected");
  }
}
